#ifdef RCSID
static char RCSid[] =
"$Header: d:/cvsroot/tads/TADS2/mac/OS0TC_MA.C,v 1.2 1999/05/17 02:52:18 MJRoberts Exp $";
#endif

/* Copyright (c) 1992 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  os0tc.c - os mainline for tads2 compiler
Function
  invokes compiler from operating system command line
Notes
  none
Modified
  04/11/99 CNebel        - Explicitly set font when creating list.
  04/02/92 MJRoberts     - creation
*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <Types.h>
//#include <OSUtils.h>
#include <ToolUtils.h>
#include <Fonts.h>
#include <MacWindows.h>
#include <Dialogs.h>
#include <Scrap.h>
#include <Files.h>
#include <StandardFile.h>
#include <Lists.h>
#include <Resources.h>
#include <SegLoad.h>
#include <Menus.h>
#include <Events.h>
#include <TextEdit.h>
#include <Devices.h>
#include <AppleEvents.h>
#include <Errors.h>
#include <NumberFormatting.h>
#include <Sound.h>
#include <TextUtils.h>
#include <Folders.h>

#include "tcd.h"
#include "linf.h"
#include "ossmac.h"
#include "cmd.h"
#include "configs.h"

/*QDGlobals qd;*/

/* remember whether any options have changed */
static int options_changed = 0;
static bool gsQuitting = false;

/*
   memory allocator - calls malloc, but tracks all blocks allocated
   so that the entire allocation chain can be blown away at the end
   of a compiler run.
*/
static void *first_block;
static int   block_count;

void *osmalloc(size_t siz)
{
        void *ret;
        Ptr   tmp;
        
        /* hold 64k in reserve for the system while allocating */
        tmp = NewPtr(0x10000);
        if (tmp == 0) return 0;
        
        /* attempt to allocate the memory */
        siz += sizeof(void *);
        ret = (void *)malloc(siz);
        if (ret)
        {
                /* link it into the block chain */
                *(void **)ret = first_block;
                first_block = ret;
                ret = (void *)(((char *)ret) + sizeof(void *));
                ++block_count;
        }
        
        /* release the reserved memory */
        DisposePtr(tmp);
        
        /* return the allocated memory */
        return(ret);
}

/* free a single chunk, removing it from our master list */
void osfree(void *p)
{
        void *target = (void *)(((char *)p) - sizeof(void *));
        void *prv;
        void *cur;
        int   found = 0;
        int   i;

        /* don't search if the list is empty */
        if (!first_block) goto done;
        
        if (target == first_block)
        {
                first_block = *(void **)target;
                found = TRUE;
                goto done;
        }
        
        if (!first_block) goto done;
        
        for (i = 0, prv = first_block, cur = *(void **)first_block ; cur ;
                 prv = cur, cur = *(void **)cur)
        {
                if (cur == target)
                {
                        found = 1;
                        *(void **)prv = *(void **)cur;
                        goto done;
                }
                
                if (i > block_count + 100)
                {
                        os_printf("\n*** internal warning: osfree chain corrupted ***");
                        goto done;
                }
        }
        
done:
        if (!found) os_printf("\n*** Internal warning: osfree search failure ***\n\n");
        else --block_count;
        free(target);
}

/* blow away all memory allocated with osmalloc() */
static void os_freeall(void)
{
        void *next_block;
        int   local_block_count = 0;
        
        for ( ; first_block ; first_block = next_block)
        {
                next_block = *(void **)first_block;
                free(first_block);
                ++local_block_count;
        }
        block_count = 0;
}

/* close all files */
static void os_closeall(void)
{
        #if NEVER
        /* The standard libraries will take care of this for us.  [CNebel 10/26/97] */
        int i;
        
        for (i = 3 ; i < FOPEN_MAX ; ++i)
                fclose(&_iob[i]);
        #endif
}

extern WindowPtr myWindow;


#define INCBUFSIZ       2048
#define MAXINC          20

static int DoCommand(long mResult);

/* resource IDs of menus */
#define appleID                 128
#define fileID                  129
#define editID                  130
#define optionsID               22229
#define windowsID               23283

/* Edit menu command indices */
#define undoCommand             1
#define cutCommand              3
#define copyCommand             4
#define pasteCommand    5
#define clearCommand    6

/* Menu indices */
#define appleM                  0
#define fileM                   1
#define editM                   2
#define optionsM                3
#define windowsM                4

#define fmOpen                  1
#define fmSaveAs                2
#define fmComp                  3
#define fmQuit                  5

#define wmMessages              1
#define wmProgress              2
#define wmError                 3

static MenuHandle myMenus[10];

static DialogPtr globdlg;
static ListHandle globlst;
static DialogPtr progdlg;               /* progress dialog */
static DialogPtr memoryDlg;             /* memory options dialog */
static DialogPtr v1Dlg;                 /* v1 options dialog */
static DialogPtr errorDlg;              /* error options dialog */
static DialogPtr versionDlg;    /* file version dialog */
static DialogPtr ppDlg;                 /* preprocessor dialog */

static struct
{
        unsigned int    setting;
        unsigned int    standard;
        char                    switchar;
//      char                    modified;
} mem_vals[] =
{
        {TCD_LABSIZ,  TCD_LABSIZ,  'g'},
        {TCD_POOLSIZ, TCD_POOLSIZ, 'p'},
        {TCD_LCLSIZ,  TCD_LCLSIZ,  'l'},
        {TCD_STKSIZ,  TCD_STKSIZ,  's'},
        {TCD_HEAPSIZ, TCD_HEAPSIZ, 'h'}
};

#define MAX_PP_SYMS  50
static char  pp_D_buf[256];
static char *pp_D_syms[MAX_PP_SYMS];
static int   pp_D_cnt;
static char  pp_U_buf[256];
static char *pp_U_syms[MAX_PP_SYMS];
static int   pp_U_cnt;

#define DEFAULT(x)  x##__default

const char              version_code__default = 'c';
static char             version_code = DEFAULT(version_code);
static Str255   stringsfile = "\p";

static void parse_pp_item(short itm, char *bufp, int *cnt, char **syms)
{
        Str255  itemText;
        char   *p;
        char   *newsym;
        int             old_cnt;

        /* get the text box contents, and convert to C-style string */
        GetControlItemTitle(ppDlg, itm, itemText);
        StringCopy_cp(bufp, itemText, 0);
        
        /* remember the original count */
        old_cnt = *cnt;
        
        /* break the string into one string per line */
        for (*cnt = 0, p = bufp; *p && *cnt < MAX_PP_SYMS; ++*cnt)
        {
                /* remember where the symbol starts */
                newsym = p;
                
                /* find the end of the line, and null-terminate it */
                while (*p && *p != '\n' && *p != '\r') ++p;
                if (*p) *p++ = '\0';
                
                /* note whether it changed */
                if (*cnt > old_cnt || strcmp(newsym, syms[*cnt]))
                                options_changed = 1;
                
                /* set the symbol pointer to the proper point in the buffer */
                syms[*cnt] = newsym;
        }
}

static void unparse_pp_item(short itm, int cnt, char **syms)
{
        Str255                  buf;
        int                             i;
        unsigned char  *p;
        
        for (i = 0, p = &buf[1]; i < cnt; ++i)
        {
                size_t l = strlen(syms[i]);
                
                if (p + l > buf + sizeof(buf)) break;
                memcpy(p, syms[i], l);
                p += l;
                *p++ = 13;
        }
        buf[0] = p - &buf[1];
        
        SetControlItemTitle(ppDlg, itm, buf);
}

static void get_pp_settings(void)
{
        if (!ppDlg) return;
        parse_pp_item(2, pp_D_buf, &pp_D_cnt, pp_D_syms);
        parse_pp_item(4, pp_U_buf, &pp_U_cnt, pp_U_syms);
}

static void set_pp_settings(void)
{
        if (!ppDlg) return;
        unparse_pp_item(2, pp_D_cnt, pp_D_syms);
        unparse_pp_item(4, pp_U_cnt, pp_U_syms);
}

static void get_version_settings(void)
{
        int             i;
        
        if (!versionDlg) return;
        for (i = 1; i <= 3; ++i)
        {
                if (GetControlItemValue(versionDlg, i) != 0)
                {
                        if (version_code != ('a' - 1) + i) options_changed = 1;
                        version_code = ('a'-1) + i;
                }
        }
}

static void set_version_settings(void)
{
        int             i;
        
        if (!versionDlg) return;
        for (i = 1; i <= 3; ++i)
        {
                if (version_code == ('a'-1) + i)
                        SetControlItemValue(versionDlg, i, 1);
        }
}

static void get_memory_settings(void)
{
        Str255          buf;
        int                     i;
        long            val;
        unsigned        newval;
        
        if (!memoryDlg) return;
        
        for (i = 0; i < 5; ++i)
        {
                GetControlItemTitle(memoryDlg, (i+1)*2, buf);
                StringToNum(buf, &val);
                newval = (unsigned) val;
                
                if (newval != mem_vals[i].setting)
                {
                        mem_vals[i].setting = newval;
                //      mem_vals[i].modified = 1;
                        options_changed = 1;
                }
        }
}

static void set_memory_settings(void)
{
        Str255  buf;
        int             i;
        
        if (!memoryDlg) return;
        
        for (i = 0; i < 5; ++i) {
                psprintf(buf, "%u", mem_vals[i].setting);
                SetControlItemTitle(memoryDlg, (i+1)*2, buf);
        }
}

const int  warning_level__default = 0;
static int warning_level = DEFAULT(warning_level);
const int  warning_abin__default = 1;
static int warning_abin = DEFAULT(warning_abin);
static char error_log_file[255];
static int  use_error_log_file;

static void get_error_settings(void)
{
        Str255  buf;
        int             i;
        
        if (!errorDlg) return;
        for (i = 3; i <= 5; ++i)
        {
                if (GetControlItemValue(errorDlg, i) != 0)
                {
                        if (warning_level != i - 3) options_changed = 1;
                        warning_level = i - 3;
                }
        }
        
        /* get the "Ambiguous Binary Operators" check mark */
        warning_abin = GetControlItemValue(errorDlg, 7);
        
        /* get text of error log filename */
        GetControlItemTitle(errorDlg, 2, buf);
        
        /* get check box for enabling/disabling error logging */
        if (GetControlItemValue(errorDlg, 1) != use_error_log_file)
                options_changed = 1;
        use_error_log_file = GetControlItemValue(errorDlg, 1);
        
        /* set error log filename */
        if (StrLength(buf) > 0 && !StringEqual_pc(buf, error_log_file))
        options_changed = 1;
        
        if (StrLength(buf) > 0)
                StringCopy_cp(error_log_file, buf, sizeof(error_log_file));
}

static void set_error_settings(void)
{
        Str255          buf;
        int                     i;
        
        if (!errorDlg) return;
        
        for (i = 3 ; i <= 5 ; ++i) {
                SetControlItemValue(errorDlg, i, i - 3 == warning_level);
        }
        
        /* set the "ambiguous binary operators" warning flag */
        SetControlItemValue(errorDlg, 7, warning_abin);
        
        /* set text of error log filename */
        StringCopy_pc(buf, error_log_file, sizeof(buf));
        SetControlItemTitle(errorDlg, 2, buf);
        
        /* set check box for enabling/disabling error logging */
        SetControlItemValue(errorDlg, 1, use_error_log_file);
        
        /* set error log file if appropriate */
        if (i && StrLength(buf) > 0)
                StringCopy_cp(error_log_file, buf, sizeof(error_log_file));
}


#define v1_dokw__default  "_do_"

static int  v1_vals[4];
static char v1_dokw[40] = DEFAULT(v1_dokw);

static void get_v1_settings(void)
{
        Str255  buf;
        int             i;
        
        if (!v1Dlg) return;
        
        for (i = 0 ; i < 4 ; ++i)
        {
                SInt16  itemVal = GetControlItemValue(v1Dlg, i+1);
                
                if (v1_vals[i] != itemVal)
                        options_changed = 1;
                v1_vals[i] = itemVal;
        }
        
        GetControlItemTitle(v1Dlg, 5, buf);
        if (!StringEqual_cp(v1_dokw, buf)) {
                options_changed = 1;
                StringCopy_cp(v1_dokw, buf, sizeof(v1_dokw));
        }
}

static void set_v1_settings(void)
{
        Str255  buf;
        int             i;
        
        if (!v1Dlg) return;
        
        for (i = 0 ; i < 4 ; ++i) {
                SetControlItemValue(v1Dlg, i+1, v1_vals[i]);
        }
        StringCopy_pc(buf, v1_dokw, sizeof(buf));
        SetControlItemTitle(v1Dlg, 5, buf);
}

static void setbutton(DialogPtr dialog, short button, short checkbox)
{
        /* enable or disable button according to check box */
        
        HiliteControlItem(dialog, button, GetControlItemValue(dialog, checkbox) ? 0 : 255);
}


static void readcommandline(char *buf, ListHandle mylist, DialogPtr mydlg)
{
        const int       MAX_ARGS = 64;
        
        int                     argc;
        char       *argv[MAX_ARGS], **argp;
        int                     i;
        char       *pp_D_p, *pp_U_p;
        
        argize(buf, &argc, argv, MAX_ARGS);
        
        /* Reset everything to its standard state.
        */
        LDelRow(0, (*mylist)->dataBounds.top, mylist);  // Include directories.
        
        pp_D_cnt = 0; pp_D_p = pp_D_buf;                        // Defines and undefines.
        pp_U_cnt = 0; pp_U_p = pp_U_buf;
        
        error_log_file[0] = '\0';                                       // Error capture file.
        use_error_log_file = false;
        
        warning_level = DEFAULT(warning_level);         // Warning level.
        warning_abin = DEFAULT(warning_abin);
        
        version_code = DEFAULT(version_code);           // Output file version.
        
        CheckItem(myMenus[optionsM], 7, false);         // Case sensitivity.
        CheckItem(myMenus[optionsM], 8, false);         // C-style operators.
        CheckItem(myMenus[optionsM], 2, false);         // Symbolic debugging info.
        CheckItem(myMenus[optionsM], 3, false);         // Statistics display.
        
        StringCopy_pp(stringsfile, "\p", sizeof(stringsfile));  // Strings file.
        
        SetControlItemValue(mydlg, 10, 1);                      // Swapping state.
        SetControlItemTitle(mydlg, 11, "\p<Default>");
        
        for (int i = 0; i < 5; i++)                                     // Memory settings.
                mem_vals[i].setting = mem_vals[i].standard;
        SetControlItemTitle(mydlg, 15, "\p");
        
        SetControlItemValue(mydlg, 17, 0);                      // Write precompiled header.
        SetControlItemTitle(mydlg, 18, "\p");

        SetControlItemValue(mydlg, 19, 0);                      // Load precompiled header.
        SetControlItemTitle(mydlg, 20, "\p");
        
        for (int i = 0; i < 4; i++)                                     // v1 compatibility settings.
                v1_vals[i] = false;
        strcpy(v1_dokw, DEFAULT(v1_dokw));

        /* Handle the options.
        */
        for (i = 0, argp = argv; i < argc; i++, argp++) {
                char   *arg;
                size_t  arglen;
                
                arg = *argp;
                if (arg[0] == '\0') {
                        /* Someone screwed up and gave us an empty argument -- skip it. */
                }
                else if (arg[0] == '-') {
                        switch(arg[1]) {
                                case 'i': /* Include directory. */
                                        arg = cmdarg(NULL, &argp, &i, argc, 1, NULL);
                                        arglen = strlen(arg);
                                        if (arglen) {
                                                Cell    c = {0, 0};
                                                
                                                c.v = LAddRow(1, 32767, mylist);
                                                LSetCell(arg, arglen, c, mylist);
                                        }
                                        break;
                                
                                case 'D': /* Symbol to define. */
                                        arg = cmdarg(NULL, &argp, &i, argc, 1, NULL);
                                        arglen = strlen(arg);
                                        if (pp_D_p + arglen <= pp_D_buf + sizeof(pp_D_buf)) {
                                                pp_D_syms[pp_D_cnt++] = pp_D_p;
                                                strcpy(pp_D_p, arg);
                                                pp_D_p += arglen + 1;
                                        }
                                        break;
                                
                                case 'U': /* Symbol to undefine. */
                                        arg = cmdarg(NULL, &argp, &i, argc, 1, NULL);
                                        arglen = strlen(arg);
                                        if (pp_U_p + arglen <= pp_U_buf + sizeof(pp_U_buf)) {
                                                pp_U_syms[pp_U_cnt++] = pp_U_p;
                                                strcpy(pp_U_p, arg);
                                                pp_U_p += arglen + 1;
                                        }
                                        break;
                                
                                case 'e': /* Error capture file. */
                                        arg = cmdarg(NULL, &argp, &i, argc, 1, NULL);
                                        strcpy(error_log_file, arg);
                                        use_error_log_file = true;
                                        break;
                                        
                                case 'v': /* Warning level. */
                                        arg = cmdarg(NULL, &argp, &i, argc, 1, NULL);
                                        if (isdigit(*arg) && arg[1] == '\0')
                                                warning_level = *arg - '0';
                                        else if ((*arg == '-' || *arg == '+') && strcmp(arg+1, "abin") == 0)
                                                warning_abin = (*arg == '+');
                                        break;
                                
                                case 'f': /* Output file version. */
                                        if (arg[2] == 'v') {
                                                arg = cmdarg(NULL, &argp, &i, argc, 2, NULL);
                                                if (arg[1] == '\0') {
                                                        switch (*arg) {
                                                                case 'a':
                                                                case 'b':
                                                                case 'c': version_code = *arg; break;
                                                                case '*': version_code = 'c'; break;
                                                        }
                                                }
                                        }
                                        break;
                                
                                case 'c': /* Case sensitivity. */
                                        if (strncmp(arg+1, "case", 4) == 0)
                                                CheckItem(myMenus[optionsM], 7, cmdtog(NULL, false, arg, 4, NULL));
                                        break;
                                
                                case 'C': /* C-style operators. */
                                        CheckItem(myMenus[optionsM], 8, cmdtog(NULL, false, arg, 1, NULL));
                                        break;
                                
                                case 'd': /* Symbolic debugging info. */
                                        if (strcmp(arg+1, "ds") == 0)
                                                CheckItem(myMenus[optionsM], 2, cmdtog(NULL, false, arg, 2, NULL));
                                        break;
                                
                                case 's': /* Statistics display. */
                                        CheckItem(myMenus[optionsM], 3, cmdtog(NULL, false, arg, 1, NULL));
                                        break;
                                
                                case 'F': /* Strings file. */
                                        if (arg[2] == 's')
                                                StringCopy_pc(stringsfile, cmdarg(NULL, &argp, &i, argc, 2, NULL), sizeof(stringsfile));
                                        break;
                                
                                case 't': /* Swapping state. */
                                        if (arg[2] == '-') {
                                                SetControlItemValue(mydlg, 10, 0);
                                                SetControlItemTitle(mydlg, 11, "\p");
                                        }
                                        else if (arg[2] == '+') {
                                                SetControlItemValue(mydlg, 10, 1);
                                                SetControlItemTitle(mydlg, 11, "\p<Default>");
                                        }
                                        else if (arg[2] == 'f') {
                                                arg = cmdarg(NULL, &argp, &i, argc, 2, NULL);
                                                if (arg[0]) {
                                                        Str255  swapname;
                                                        
                                                        StringCopy_pc(swapname, arg, sizeof(swapname));
                                                        SetControlItemValue(mydlg, 10, 1);
                                                        SetControlItemTitle(mydlg, 11, swapname);
                                                }
                                        }
                                        break;
                                
                                case 'm': /* Memory settings. */
                                        switch(arg[2]) {
                                                case 'g': mem_vals[0].setting = atoi(cmdarg(NULL, &argp, &i, argc, 2, NULL)); break;
                                                case 'p': mem_vals[1].setting = atoi(cmdarg(NULL, &argp, &i, argc, 2, NULL)); break;
                                                case 'l': mem_vals[2].setting = atoi(cmdarg(NULL, &argp, &i, argc, 2, NULL)); break;
                                                case 's': mem_vals[3].setting = atoi(cmdarg(NULL, &argp, &i, argc, 2, NULL)); break;
                                                case 'h': mem_vals[4].setting = atoi(cmdarg(NULL, &argp, &i, argc, 2, NULL)); break;
                                                default: {
                                                        Str255  cachesize;
                                                        
                                                        arg = cmdarg(NULL, &argp, &i, argc, 1, NULL);
                                                        StringCopy_pc(cachesize, arg, sizeof(cachesize));
                                                        SetControlItemTitle(mydlg, 15, cachesize);
                                                        break;
                                                }
                                        }
                                        break;
                                
                                case 'w': /* Write precompiled header. */
                                        arg = cmdarg(NULL, &argp, &i, argc, 1, NULL);
                                        if (arg[0]) {
                                                Str255          headername;
                                                
                                                StringCopy_pc(headername, arg, sizeof(headername));
                                                SetControlItemValue(mydlg, 17, 1);
                                                SetControlItemTitle(mydlg, 18, headername);
                                        }
                                        break;
                                
                                case 'l': /* Load precompiled header. */
                                        arg = cmdarg(NULL, &argp, &i, argc, 1, NULL);
                                        if (arg[0]) {
                                                Str255          headername;
                                                
                                                StringCopy_pc(headername, arg, sizeof(headername));
                                                SetControlItemValue(mydlg, 19, 1);
                                                SetControlItemTitle(mydlg, 20, headername);
                                        }
                                        break;
                                
                                case '1': /* v1 compatibility settings. */
                                        switch (arg[2]) {
                                                case 'e': v1_vals[0] = cmdtog(NULL, v1_vals[0], arg, 2, NULL); break;
                                                case 'k': v1_vals[1] = cmdtog(NULL, v1_vals[1], arg, 2, NULL); break;
                                                case 'a': v1_vals[2] = cmdtog(NULL, v1_vals[2], arg, 2, NULL); break;
                                                case 'd':
                                                        v1_vals[3] = true;
                                                        arg = cmdarg(NULL, &argp, &i, argc, 1, NULL);
                                                        strcpy(v1_dokw, arg);
                                                        break;
                                                case '\0':
                                                        v1_vals[0] = !v1_vals[0];
                                                        v1_vals[1] = !v1_vals[1];
                                                        v1_vals[2] = !v1_vals[2];
                                                        break;
                                        }
                                        break;
                                
                                case 'Z': /* Run-time argument checking. */
                                        switch (arg[2]) {
                                                case 'a': v1_vals[2] = cmdtog(NULL, v1_vals[2], arg, 2, NULL); break;
                                        }
                                        break;
                                
                                case 'o': /* Output file. */
                                        /* XXX: Ignored, but it shouldn't be.  There's no control for this option. */
                                        break;
                                
                                case 'p': /* Pause at end. */
                                        /* Ignored. */
                                        break;
                        }               
                }
                else {
                        /* If it's not argv[0], and it doesn't begin with '-', it must be the input file. */
                        if (i != 0) {
                                Str255          gamename;
                                
                                StringCopy_pc(gamename, arg, sizeof(gamename));
                                SetControlItemTitle(mydlg, 1, gamename);
                                
                                /* Enable the "Compile" button if the name isn't empty. */
                                if (StrLength(gamename) > 0) {
                                        EnableControlItem(mydlg, 8);
                                        EnableItem(myMenus[fileM], fmComp);
                                }
                        }
                }
        }
        
        set_error_settings();
        set_memory_settings();
        set_v1_settings();
}

static bool writecommandline(DialogPtr mydlg, ListHandle mylist, writeoptions_t &opts, bool quote)
{
        #define P(s)  s[0], &s[1]
        
        Str255  itemText;
        Cell    cell;
        short   markChar;
        int             i;
        
        opts.write("tadsc");
        
        /* Include directories. */
        cell.v = cell.h = 0;
        while (cell.v < (*mylist)->dataBounds.bottom)
        {
                char    includefile[256];
                short   siz;
                
                siz = 256;
                LGetCell(includefile, &siz, cell, mylist);
                opts.write(quote ? "-i\"%.*s\"" : "-i%.*s", siz, includefile);
                cell.v++;
        }
        
        /* Defines and undefines. */
        get_pp_settings();
        for (i = 0; i < pp_D_cnt; ++i)
        {
                opts.write("-D");
                opts.write("%s", pp_D_syms[i]);
        }
        
        for (i = 0; i < pp_U_cnt; ++i)
        {
                opts.write("-U");
                opts.write("%s", pp_U_syms[i]);
        }
        
        /* Error capture file. */
        get_error_settings();
        if (use_error_log_file)
        {
                opts.write("-e");
                opts.write("%s", error_log_file);
        }
        
        /* Warning level. */
        if (warning_level != DEFAULT(warning_level))
                opts.write("-v%c", '0' + warning_level);
        
        if (warning_abin != DEFAULT(warning_abin))
                opts.write("-v%cabin", warning_abin ? '+' : '-');
        
        /* Output file version. */
        get_version_settings();
        if (version_code != DEFAULT(version_code))
                opts.write("-fv%c", version_code);
        
        /* Case sensitivity. */
        GetItemMark(myMenus[optionsM], 7, &markChar);
        if (markChar)
                opts.write("-case%c", markChar ? '+' : '-');
        
        /* C-style operators. */
        GetItemMark(myMenus[optionsM], 8, &markChar);
        if (markChar)
                opts.write("-C%c", markChar ? '+' : '-');
        
        /* Symbolic debugging info. */
        GetItemMark(myMenus[optionsM], 2, &markChar);
        if (markChar)
                opts.write("-ds");
                        
        /* Statistics display. */
        GetItemMark(myMenus[optionsM], 3, &markChar);
        if (markChar)
                opts.write("-s");
        
        /* Strings file. */
        if (StrLength(stringsfile) > 0) {
                opts.write("-Fs");
                opts.write(quote ? "\"%.*s\"" : "%.*s", P(stringsfile));
        }
        
        /* Swapping state. */
        if (GetControlItemValue(mydlg, 10) != 0)
        {
                /* swapping is enabled */
                Str255  swapname;
                
                GetControlItemTitle(mydlg, 11, swapname);
                /* if it contains "<Default>", just add "-t+" */
                if (StringEqual_pc(swapname, "<Default>"))
                        // This is the default state -- write nothing.
                        ; //opts.write("-t+");
                else {
                        opts.write("-tf");
                        opts.write(quote ? "\"%.*s\"" : "%.*s", P(swapname));
                }
        }
        else
        {
                /* swapping is disabled */
                opts.write("-t-");
        }
        
        /* Memory settings. */
        get_memory_settings();
        for (i = 0; i < 5; ++i)
        {
                if (mem_vals[i].setting != mem_vals[i].standard)
                        opts.write("-m%c%u", mem_vals[i].switchar, mem_vals[i].setting);
        }
        
        // max cache size.
        GetControlItemTitle(mydlg, 15, itemText);
        if (StrLength(itemText))
                opts.write("-m%.*s", P(itemText));
        
        /* Write precompiled header. */
        if (GetControlItemValue(mydlg, 17))
        {
                GetControlItemTitle(mydlg, 18, itemText);
                if (itemText[0])
                {
                        opts.write("-w");
                        opts.write(quote ? "\"%.*s\"" : "%.*s", P(itemText));
                }
        }
        
        /* Load precompiled header. */
        if (GetControlItemValue(mydlg, 19))
        {
                GetControlItemTitle(mydlg, 20, itemText);
                if (itemText[0])
                {
                        opts.write("-l");
                        opts.write(quote ? "\"%.*s\"" : "%.*s", P(itemText));
                }
        }
        
        /* v1 compatibility settings. */
        get_v1_settings();
        for (i = 0; i < 3; ++i)
                if (v1_vals[i] != 0)
                        opts.write("-1%c%c", "eka"[i], v1_vals[i] ? "++-"[i] : "--+"[i]);
        
        if (v1_vals[3])
                opts.write("-1d%s", v1_dokw);

        /* Add filename; return whether or not there's a valid game name. */
        GetControlItemTitle(mydlg, 1, itemText);
        if (StrLength(itemText) != 0) {
                opts.write(quote ? "\"%.*s\"" : "%.*s", P(itemText));
                return true;
        }
        else
                return false;
        
        #undef P
}

static int getconfig(const char *fname, ListHandle mylist, DialogPtr mydlg)
{
        char            buf[LINE_BUF_SIZE];
        int                     err;
        
        if ((err = readconfig(fname, "tc", "tadsc", buf, LINE_BUF_SIZE)) == noErr)
                readcommandline(buf, mylist, mydlg);
        
        return err;
}

static int putconfig(const char *fname, ListHandle mylist, DialogPtr mydlg)
{
        char    buf[LINE_BUF_SIZE];
        
        writecommandline(mydlg, mylist, wrtoptstring_t(buf), true);
        return writeconfig(fname, buf);
}


int topEvent(DialogPtr mydlg, ListHandle mylist)
{
        EventRecord             myEvent;
        WindowPtr               whichWindow;
        Boolean                 gotevent;
        char               *argv[50];
        int                             argc;
        static int              compiler_running;
        Str255                  gamename;
        Cell                    cell;
        static long             orig_cache_size;
        static bool             got_orig_cache_size;
        Str255                  buf;
        
        if (!mydlg) mydlg = globdlg;
        if (!mylist) mylist = globlst;
        
        /* note original setting of cache size, so we can compare it later */
        if (!got_orig_cache_size)
        {
                GetControlItemTitle(mydlg, 15, buf);
                StringToNum(buf, &orig_cache_size);
                got_orig_cache_size = true;
        }
        
        /*
           maintain buttons
        */
        
        /* turn on "Remove" button only if something in list is hilited */
        cell.v = cell.h = 0;
        HiliteControlItem(mydlg, 7, LGetSelect(TRUE, &cell, mylist) ? 0 : 255);
        
        /* turn on select buttons only when check boxes are checked */
        setbutton(mydlg, 21, 17);
        setbutton(mydlg, 22, 19);
        setbutton(mydlg, 13, 10);
        setbutton(mydlg, 23, 10);
        if (errorDlg) setbutton(errorDlg, 6, 1);
        
        SystemTask();
        gotevent = WaitNextEvent(everyEvent, &myEvent, 0L, 0L);
        if (!(myEvent.modifiers & cmdKey) && IsDialogEvent(&myEvent))
        {
                DialogPtr       dlg;
                short           i, itemHit;
                char            incbuf[INCBUFSIZ];
                Point           oldwhere;
                char            csbuf[255];
                        
                oldwhere = myEvent.where;
                        
                if (myEvent.what == updateEvt
                        && (WindowPtr)myEvent.message == (WindowPtr)mydlg)
                {
                        LUpdate(((GrafPtr)mydlg)->visRgn, mylist);
                        SetPort((GrafPtr)mydlg);

                        /* draw box around filenames */
                        drawbox(mydlg, 1);
                        drawbox(mydlg, 11);
                        drawbox(mydlg, 20);
                        drawbox(mydlg, 18);
                        drawbox(mydlg, 5);
                }
                
                if (myEvent.what == updateEvt && (WindowPtr)myEvent.message == (WindowPtr)errorDlg)
                {
                        /* draw box around filename */
                        SetPort((GrafPtr)errorDlg);
                        drawbox(errorDlg, 2);
                }

                if (DialogSelect(&myEvent, &dlg, &itemHit))
                {
                        if (dlg == progdlg)
                        {
                                if (itemHit == 1) return(2);  /* cancel button */
                                return(1);
                        }
                        
                        if (dlg == memoryDlg)
                        {
                                return(1);
                        }
                        
                        if (dlg == ppDlg) return 1;
                        
                        if (dlg == versionDlg)
                        {
                                switch(itemHit)
                                {
                                        case 1:
                                        case 2:
                                        case 3:
                                                for (i = 1 ; i <= 3 ; ++i)
                                                {
                                                        SetControlItemValue(dlg, i, (i == itemHit ? TRUE : FALSE));
                                                        options_changed = 1;
                                                }
                                                break;
                                }
                                
                                return(1);
                        }
                        
                        if (dlg == v1Dlg)
                        {
                                switch(itemHit)
                                {
                                        case 1:
                                        case 2:
                                        case 3:
                                        case 4: {
                                                ToggleControlItemValue(dlg, itemHit);
                                                options_changed = 1;
                                                break;
                                        }
                                }
                                
                                return(1);
                        }
                        
                        if (dlg == errorDlg)
                        {
                                switch(itemHit)
                                        {
                                        case 1:    /* checkbox for writing errors to a file */
                                        case 7: {  /* check for enabling TADS-357 error */
                                                ToggleControlItemValue(dlg, itemHit);
                                                options_changed = 1;
                                                
                                                /* keep going only if this is the error file checkbox */
                                                if (itemHit != 1) break;
                                                
                                                /* see if we need to select a file for the first time */
                                                GetControlItemTitle(dlg, 2, (StringPtr) csbuf);
                                                if (csbuf[0] != 0) break;
                                                /* FALLTHROUGH */
                                        }
                                        case 6: {       /* select file */
                                                Str255 fname;
                                                
                                                if (os0askfile("Error logging file:", fname, 'TEXT', TRUE))
                                                        break;
                                                SetControlItemTitle(dlg, 2, fname);
                                                options_changed = 1;
                                                break;
                                        }
                                        case 3:
                                        case 4:
                                        case 5:         /* radio buttons */
                                                for (i = 3 ; i <= 5 ; ++i)
                                                {
                                                        SetControlItemValue(dlg, i, (i == itemHit ? TRUE : FALSE));
                                                        options_changed = 1;
                                                }
                                                break;
                                }
                                
                                /* all done with this event */
                                return(1);
                        }
                        
                        if (dlg == mydlg)
                        {
                                switch(itemHit)
                                {
                                        case 3: {  /* "Select" button - select game name */
                                                Str255  gamename;
                                                
                                                if (os0askfile("Game Source File", gamename, 'TEXT', 0))
                                                        break;
                                                SetControlItemTitle(mydlg, 1, gamename);
                                                
                                                /* we can turn on the "compile" button now */ 
                                                EnableControlItem(mydlg, 8);
                                                EnableItem(myMenus[fileM], fmComp);
                                                options_changed = 1;
                                                break;
                                        }
                                        case 6:   /* "Add" button - add include folder */
                                                if (osaskdir(incbuf, INCBUFSIZ) == noErr) {
                                                        cell.v = LAddRow(1, 32767, mylist);
                                                        cell.h = 0;
                                                        LSetCell(incbuf, (short) strlen(incbuf), cell, mylist);
                                                        LDraw(cell, mylist);
                                                        options_changed = 1;
                                                }
                                                break;
                                        
                                        case 7:   /* "Remove" button - remove an include folder */
                                                cell.v = cell.h = 0;
                                                while (LGetSelect(TRUE, &cell, mylist))
                                                {
                                                        LDelRow(1, cell.v, mylist);
                                                        options_changed = 1;
                                                }
                                                break;
                                        
                                        case 8:   /* "Compile" button */
                                        start_compile:
                                                if (compiler_running) break;
                                                
                                                argc = 0;
                                                if (!writecommandline(mydlg, mylist, wrtoptargv_t(incbuf, argc, argv), false))
                                                        break;
                                                
                                                /* set up some info for recursive os_yield() calls */
                                                globdlg = mydlg;
                                                globlst = mylist;
                                                
                                                /* disallow certain operations while the compiler is active */
                                                compiler_running = TRUE;
                                                DisableItem(myMenus[fileM], fmQuit);
                                                DisableItem(myMenus[fileM], fmComp);
                                                EnableItem(myMenus[windowsM], wmProgress);
                                                DisableControlItem(mydlg, 8);
                                                DisableControlItem(mydlg, 9);
                                                #if DEBUG
                                                for (i = 1; i < argc; ++i) os_printf("%s\n", argv[i]);
                                                {
                                                        char            buf[2048];
                                                        
                                                        writecommandline(mydlg, mylist, wrtoptstring_t(buf), false);
                                                        os_printf("%s\n", buf);
                                                }
                                                #endif  
                                                /* put up the progress modeless dialog box */
                                                ShowWindow(myWindow);
                                                SelectWindow(myWindow);
                                                progdlg = GetNewDialog(9585, NULL, (WindowPtr) -1L);

                                                /* go run the compiler */
                                                {
                                                        UInt32  start, elapsed;
                                                        
                                                        start = TickCount();
                                                        if (os0main(argc, argv, tcdmain, "", "" /*"config.tc"*/) == 0) {
                                                                elapsed = (TickCount() - start) / 60;
                                                                os_printf("*** Compilation completed (%d:%.2d elapsed) ***\n\n", elapsed / 60, elapsed % 60);
                                                        }
                                                        else
                                                                os_printf("*** Compilation terminated ***\n\n");
                                                }
                                                
                                                /* get rid of progress dialog */
                                                DisposeDialog(progdlg);
                                                progdlg = 0;
                                                
                                                /* done with compile - turn on 'quit' and 'compile' again */
                                                EnableItem(myMenus[fileM], fmQuit);
                                                EnableItem(myMenus[fileM], fmComp);
                                                DisableItem(myMenus[windowsM], wmProgress);
                                                EnableControlItem(mydlg, 8);
                                                EnableControlItem(mydlg, 9);
                                                compiler_running = FALSE;
                                                
                                                /* blow away all memory allocated during compilation */
                                                os_freeall();
                                                
                                                /* likewise, close all open files */
                                                os_closeall();
                                                break;
                                                
                                        case 9:  /* "Quit" button */
                                                if (!compiler_running) goto do_quit;
                                                break;
                                                
                                        case 5:  /* include file list box */
                                                SetPort((WindowPtr)mydlg);
                                                GlobalToLocal(&oldwhere);
                                                LClick(oldwhere, myEvent.modifiers, mylist);
                                                break;
                                                
                                        case 10:  /* "Allow Swapping" check box */
                                        case 12:  /* "Use default swap file" check box */
                                        case 16:  /* "v1 compatibility mode" check box */
                                        case 19:  /* "load precompiled header" check box */
                                        case 17:  /* "generate precompiled header" check box */
                                        {
                                                options_changed = 1;
                                                
                                                if (ToggleControlItemValue(mydlg, itemHit) && (itemHit == 19 || itemHit == 17))
                                                {
                                                        /* automatically prompt for a file upon first selection */
                                                        GetControlItemTitle(mydlg, itemHit+1, gamename);
                                                        if (StrLength(gamename) == 0)
                                                        {
                                                                if (itemHit == 19) goto selectLoad;
                                                                else goto selectGen;
                                                        }
                                                }
                                                break;
                                        }
                                        
                                        case 13:  /* "Select" (swap file) button */
                                                if (os0askfile("Swap File", gamename, 'TEXT', 1))
                                                        break;
                                                SetControlItemTitle(mydlg, 11, gamename);
                                                options_changed = 1;
                                                break;
                                
                                        case 23:  /* "Automatic Selection" (swap file) button */
                                                SetControlItemTitle(mydlg, 11, "\p<Default>");
                                                options_changed = 1;
                                                break;
                                
                                        case 21:  /* "Select" (generate precompiled header) button */
                                        selectGen:
                                                if (os0askfile("Precompiled Header", gamename, 'TADG', 1))
                                                        break;
                                                SetControlItemTitle(mydlg, 18, gamename);
                                                options_changed = 1;
                                                break;
                                                
                                        case 22:  /* "Select" (load precompiled header) button */
                                        selectLoad:
                                                if (os0askfile("Precompiled Header", gamename, 'TADG', 0))
                                                        break;
                                                SetControlItemTitle(mydlg, 20, gamename);
                                                options_changed = 1;
                                                break;
                                }
                        }
                }
                
                return(1);
        }
        
        if (gotevent)
        {
                long menuCmd;
                
                switch (myEvent.what)
                {
                        case mouseDown:
                                switch (FindWindow( myEvent.where, &whichWindow ))
                                {
                                        case inDesk:
                                                SysBeep(10);
                                                break;
                                                        
                                        case inGoAway:
                                                if (!TrackGoAway(whichWindow, myEvent.where)) break;
                                                if (memoryDlg && whichWindow == (WindowPtr)memoryDlg)
                                                {
                                                        get_memory_settings();
                                                        DisposeDialog(memoryDlg);
                                                        memoryDlg = 0;
                                                }
                                                else if (v1Dlg && whichWindow == (WindowPtr)v1Dlg)
                                                {
                                                        get_v1_settings();
                                                        DisposeDialog(v1Dlg);
                                                        v1Dlg = 0;
                                                }
                                                else if (errorDlg && whichWindow == (WindowPtr)errorDlg)
                                                {
                                                        get_error_settings();
                                                        DisposeDialog(errorDlg);
                                                        errorDlg = 0;
                                                }
                                                else if (versionDlg && whichWindow == (WindowPtr)versionDlg)
                                                {
                                                        get_version_settings();
                                                        DisposeDialog(versionDlg);
                                                        versionDlg = 0;
                                                }
                                                else if (ppDlg && whichWindow == (WindowPtr)ppDlg)
                                                {
                                                        get_pp_settings();
                                                        DisposeDialog(ppDlg);
                                                        ppDlg = 0;
                                                }
                                                else if (whichWindow == myWindow && !compiler_running)
                                                        HideWindow(whichWindow);
                                                break;

                                        case inMenuBar:
                                                menuCmd = MenuSelect(myEvent.where);
                                        do_menu:
                                                if (HiWord(menuCmd) == fileID)
                                                {
                                                        HiliteMenu(0);
                                                        if (LoWord(menuCmd) == fmComp) goto start_compile;
                                                        if (LoWord(menuCmd) == fmQuit) goto do_quit;
                                                }
                                                return(DoCommand(menuCmd));
                                        
                                        case inSysWindow:
                                                SystemClick( &myEvent, whichWindow );
                                                break;
                                        
                                        case inDrag:
                                                if (ours(whichWindow) || whichWindow == (WindowPtr)mydlg
                                                        || (progdlg && whichWindow == (WindowPtr)progdlg)
                                                        || (memoryDlg && whichWindow == (WindowPtr)memoryDlg)
                                                        || (v1Dlg && whichWindow == (WindowPtr)v1Dlg)
                                                        || (errorDlg && whichWindow == (WindowPtr)errorDlg)
                                                        || (versionDlg && whichWindow == (WindowPtr)versionDlg)
                                                        || (ppDlg && whichWindow == (WindowPtr)ppDlg))
                                                {
                                                        static Rect dragRect = (*GetGrayRgn())->rgnBBox;
                                                        
                                                        /* oldfront = whichWindow; */
                                                        DragWindow(whichWindow, myEvent.where, &dragRect);
                                                }
                                                break;
                                        
                                        case inContent:
                                                if (whichWindow != FrontWindow())
                                                {
                                                        SelectWindow(whichWindow);
                                                        /* oldfront = whichWindow; */
                                                }
                                                else if (whichWindow == (WindowPtr)mydlg)
                                                {
                                                        LClick(myEvent.where, myEvent.modifiers, mylist);
                                                }
                                                else if (ours(whichWindow))
                                                        return(DoContent(whichWindow, &myEvent));
                                                break;
                                                
                                        case inGrow:
                                                if (ours(whichWindow)) MyGrowWindow(whichWindow, myEvent.where);
                                                break;

                                        default:
                                                break;
                                }
                                break;
                                        
                        case activateEvt:
                        {
                                Rect r;
                                extern TEHandle TEH;
                                extern ControlHandle vScroll;

                                if (ours(whichWindow = (WindowPtr)myEvent.message)) {
                                        r = whichWindow->portRect;
                                        r.top = r.bottom - (kScrollBarWidth+1);
                                        r.left = r.left - (kScrollBarWidth+1);
                                        InvalRect(&r);
                                        if ( myEvent.modifiers & activeFlag ) {
                                                TEActivate( TEH );
                                                ShowControl( vScroll );
                                                TEFromScrap();
                                        }
                                        else {
                                                TEDeactivate(TEH);
                                                HideControl( vScroll );
                                                ZeroScrap();
                                                TEToScrap();
                                        }
                                }
                                break;
                        }

                        case updateEvt:
                                if (ours(whichWindow = (WindowPtr)myEvent.message))
                                        UpdateWindow(whichWindow);
                                break;
                                
                        case keyDown:
                        case autoKey:
                        {
                                char theChar = (myEvent.message & charCodeMask);
                                
                                if ((myEvent.modifiers & cmdKey) && theChar == '.') return(2);
                                if (myEvent.modifiers & cmdKey)
                                {
                                        menuCmd = MenuKey(theChar);
                                        goto do_menu;
                                }
                                break;
                        }
                        
                        case kHighLevelEvent: {
                                OSErr   err;
                                
                                globdlg = mydlg;
                                globlst = mylist;
                                err = AEProcessAppleEvent(&myEvent);
                                assert(err == noErr);
                                break;
                        }

                        default: ;
                }
        }
        
        if (gsQuitting) {
        do_quit:
                /* get all options out of dialogs */
                get_v1_settings();
                get_error_settings();
                get_version_settings();
                get_memory_settings();
                get_pp_settings();
                
                /* Save options to master prefs file without asking. */
                putconfig(NULL, mylist, mydlg);
                
                return 0;
        }
        
        return 1;
}

void os_progress(linfdef *linf)
{
        static char             prevlinfnam[256] = "";
        static UInt32   lastUpdateTime;
        UInt32                  now;
        const UInt32    threshold = 15;  // Minimum ticks between updates.
        Str255                  buf;
        bool                    namechanged;

        if (!progdlg) return; /* spurious entry */

        now = TickCount();
        if (now > lastUpdateTime + threshold || (namechanged = strcmp(linf->linfnam, prevlinfnam)) != 0) {
                lastUpdateTime = now;
                
                if (namechanged) {
                        StringCopy_cc(prevlinfnam, linf->linfnam, 256);
                        StringCopy_pc(buf, linf->linfnam, sizeof(buf));
                        SetControlItemTitle(progdlg, 3, buf);
                }
                
                psprintf(buf, "%u", linf->linfnum);
                SetControlItemTitle(progdlg, 5, buf);
        }
}

static void SetUpMenus(void)
{
        int i;
        
        myMenus[appleM] = GetMenu(appleID);
        AppendResMenu(myMenus[appleM], 'DRVR');
        
        myMenus[fileM] = GetMenu(fileID);
        myMenus[editM] = GetMenu(editID);
        myMenus[optionsM] = GetMenu(optionsID);
        myMenus[windowsM] = GetMenu(windowsID);
        for (i = appleM ; i <= windowsM ; i++)
                InsertMenu(myMenus[i], 0);
        DrawMenuBar();

        DisableItem(myMenus[editM], undoCommand);
        DisableItem(myMenus[editM], cutCommand);
        DisableItem(myMenus[editM], copyCommand);
        DisableItem(myMenus[editM], clearCommand);
        DisableItem(myMenus[editM], pasteCommand);
}

static int DoCommand(long mResult)
{
        int             theItem;
        Str255  name;
        char   *newcmd = NULL;
        long    cmdlen = 0;
        
        theItem = LoWord( mResult );
        switch (HiWord(mResult))
        {
                case appleID:
                        if (theItem == 1) /* about */
                        {
                                DialogPtr       mydlg;
                                short           itemHit;
                                
                                mydlg = GetNewDialog(7768, NULL, (WindowPtr) -1L);
                                do
                                {
                                        ModalDialog(NULL, &itemHit);
                                } while (itemHit != 1);
                                DisposeDialog(mydlg);
                        }
                        else
                        {
                                GetMenuItemText(myMenus[appleM], theItem, name);
                                OpenDeskAcc(name);
                        }
                        break;
                        
                
                case fileID:
                                switch (theItem)
                                {
                                        #if 0
                                        // Compile and Quit are handled in topEvent.
                                        case fmComp:
                                                return 0;
                                        
                                        case fmQuit:
                                                return 0;
                                        #endif
                                        
                                        case fmOpen: {
                                                char            fname[256];
                                                
                                                if (os0askfile("Open compiler settings file", fname, 256, 'TEXT', false) == 0)
                                                        getconfig(fname, globlst, globdlg);
                                                
                                                break;
                                        }
                                        
                                        case fmSaveAs: {
                                                char            fname[256];
                                                
                                                if (os0askfile("Save compiler settings to", fname, 256, 'TEXT', true) == 0)
                                                        putconfig(fname, globlst, globdlg);
                                                
                                                break;
                                        }
                                }
                                break;
                
                
                case optionsID:
                        switch(theItem)
                        {
                                case 1:
                                        if (v1Dlg)
                                                SelectWindow((WindowPtr)v1Dlg);
                                        else
                                        {
                                                v1Dlg = GetNewDialog(15901, NULL, (WindowPtr) -1L);
                                                set_v1_settings();
                                        }
                                        break;
                                
                                case 2:
                                case 3:
                                case 7:
                                case 8:
                                {
                                        short mark;
                                        
                                        GetItemMark(myMenus[optionsM], theItem, &mark);
                                        SetItemMark(myMenus[optionsM], theItem, mark ? 0 : 0x12);
                                        options_changed = 1;
                                        break;
                                }
                                
                                case 4:
                                        if (memoryDlg)
                                                SelectWindow((WindowPtr)memoryDlg);
                                        else
                                        {
                                                memoryDlg = GetNewDialog(13919, NULL, (WindowPtr) -1L);
                                                set_memory_settings();
                                        }
                                        break;
                                
                                case 5:
                                        if (errorDlg)
                                                SelectWindow((WindowPtr)errorDlg);
                                        else
                                        {
                                                errorDlg = GetNewDialog(9407, NULL, (WindowPtr) -1L);
                                                set_error_settings();
                                        }
                                        break;
                                
                                case 6:
                                        if (versionDlg)
                                                SelectWindow((WindowPtr)versionDlg);
                                        else
                                        {
                                                versionDlg = GetNewDialog(28156, NULL, (WindowPtr) -1L);
                                                set_version_settings();
                                        }
                                        break;
                                
                                case 9:
                                        if (ppDlg)
                                                SelectWindow((WindowPtr)ppDlg);
                                        else
                                        {
                                                ppDlg = GetNewDialog(24912, NULL, (WindowPtr) -1L);
                                                set_pp_settings();
                                        }
                                        break;
                        }
                        break;
                
                case windowsID:
                        switch(theItem)
                        {
                                case wmMessages:
                                        ShowWindow(myWindow);
                                        SelectWindow(myWindow);
                                        break;
                                        
                                case wmProgress:
                                        if (progdlg)
                                                        SelectWindow((WindowPtr)progdlg);
                                        break;
                                        
                                case wmError:
                                        if (errorDlg)
                                                SelectWindow((WindowPtr)errorDlg);
                                        break;
                        }
                        break;
        }

        HiliteMenu(0);
        return(1);
}


OSErr HandleOpenApp()
{
        return noErr;
}

//static FSSpec startDocument;

OSErr HandleOpenDocument(const FSSpec *doc)
{
        FInfo   fInfo;
        
        FSpGetFInfo(doc, &fInfo);
        if (fInfo.fdType == 'TEXT') {
                OSErr   err;
                char    fPath[256];
                
                //DPRINTF(("Got a document named \"%P\".\n", doc->name));
                //startDocument = *doc;
                if ((err = fsspectopath(doc, fPath)) == noErr) {
                        getconfig(fPath, globlst, globdlg);
                        return noErr;
                }
                else
                        return err;
        }
        else {
                //DPRINTF(("Got document of type %.4s -- not handled.\n", &fInfo.fdType));
                return errAEEventNotHandled;
        }
}

void HandleQuitApp()
{
        gsQuitting = true;
}

void main(void)
{
        DialogPtr       mydlg;
        ListHandle      mylist;
        FSSpec          appFSSpec;
        char            apppath[256];
        Str255          itemText;

        InitToolbox();
        GetApplicationInfo(NULL, &appFSSpec);
        dirtopath(appFSSpec.vRefNum, appFSSpec.parID, apppath);
//      HandleAllHighLevelEvents();
        
        SetUpMenus();
        SetUpWindows();

        /* set up dialog and include file list box */
        mydlg = GetNewDialog(19587, NULL, (WindowPtr) -1L);
        {
                Rect    dataBounds;
                Point   cSize;
                Rect    itemRect;
                
                TextFont(systemFont);
                TextSize(0);
                GetControlItemRect(mydlg, 5, &itemRect);
                itemRect.right -= 15;  /* for scroll bar */
                dataBounds.top = 0; dataBounds.left = 0; dataBounds.bottom = 0;
                dataBounds.right = 1;
                cSize.h = cSize.v = 0;
                mylist = LNew(&itemRect, &dataBounds, cSize, 0, (WindowPtr) mydlg, TRUE, FALSE, FALSE, TRUE);
        /*      LAddColumn(1, 0, mylist);*/
        }

        /* try reading config; if not found, default stuff */
        if (getconfig(NULL, mylist, mydlg))
        {
                Cell    cell;
                
                /* add a row with the current directory to the include path */
                cell.v = LAddRow(1, 32767, mylist);
                cell.h = 0;
                LSetCell(apppath, strlen(apppath), cell, mylist);
                
                /* set up swap file name */
                SetControlItemTitle(mydlg, 11, "\p<Default>");
                
                /* turn on swapping by default */
                SetControlItemValue(mydlg, 10, 1);
                
                /* case sensitivity on by default */
                SetItemMark(myMenus[optionsM], 7, 0x12);
        }
        
        GetControlItemTitle(mydlg, 1, itemText);
        
        if (itemText[0] == 0)
        {
                /* make "Compile" button inactive until we get a filename */
                DisableControlItem(mydlg, 8);
                DisableItem(myMenus[fileM], fmComp);
        }
        DisableItem(myMenus[windowsM], wmProgress);

        while (topEvent(mydlg, mylist))
                ;

        XXX_FixStandAlone();
}
