/*
==========================================================================================
==
==	File:		mstring.h
==
==	Contains:	Pascal- and C-string manipulation routines
==
==	Copyright:	 1998 by Christopher Nebel, all rights reserved.
==
==	Change History (most recent first):
==
==		 <1>	 7/23/98	Neb		Spliced off from ossmac.h.
==
=========================================================================================
*/

/*
	These functions are modeled on strcpy and strcat: destination is the first
	parameter, source the second, and they all return the destination pointer.
	
	Destination size is measured in bytes, and includes any length or termination
	bytes.  Pass zero to assume the maximum size (256 for Pascal-style, unlimited
	for C-style.)
	
	Those functions that take a Pascal string (StringPtr, aka unsigned char *) as
	their destination will also null-terminate it, as a favor to code that
	expects to pass portions of strings to C library functions.
 */

#ifndef __MSTRING__
#define __MSTRING__

#include <stddef.h>
#include <stdarg.h>
#include <string.h>
#include <Types.h>


/* dst StringCopy(dst, src) -- copy src to dst; return dst.
*/
StringPtr StringCopy_pp(StringPtr dst, ConstStr255Param src, size_t dstSize);
StringPtr StringCopy_pc(StringPtr dst, const char *src, size_t dstSize);
char *StringCopy_cp(char *dst, ConstStr255Param src, size_t dstSize);
char *StringCopy_cc(char *dst, const char *src, size_t dstSize);

#ifdef __cplusplus
inline StringPtr StringCopy(StringPtr dst, ConstStr255Param src, size_t dstSize)
	{ return StringCopy_pp(dst, src, dstSize); }

inline StringPtr StringCopy(StringPtr dst, const char *src, size_t dstSize)
	{ return StringCopy_pc(dst, src, dstSize); }

inline char *StringCopy(char *dst, ConstStr255Param src, size_t dstSize)
	{ return StringCopy_cp(dst, src, dstSize); }

inline char *StringCopy(char *dst, const char *src, size_t dstSize)
	{ return StringCopy_cc(dst, src, dstSize); }
#endif

/* dst StringCat(dst, src) -- catenate src to dst; return dst.
*/
StringPtr StringCat_pp(StringPtr dst, ConstStr255Param src, size_t dstSize);
StringPtr StringCat_pc(StringPtr dst, const char *src, size_t dstSize);
char *StringCat_cp(char *dst, ConstStr255Param src, size_t dstSize);
char *StringCat_cc(char *dst, const char *src, size_t dstSize);

#ifdef __cplusplus
inline StringPtr StringCat(StringPtr dst, ConstStr255Param src, size_t dstSize)
	{ return StringCat_pp(dst, src, dstSize); }

inline StringPtr StringCat(StringPtr dst, const char *src, size_t dstSize)
	{ return StringCat_pc(dst, src, dstSize); }

inline char *StringCat(char *dst, ConstStr255Param src, size_t dstSize)
	{ return StringCat_cp(dst, src, dstSize); }

inline char *StringCat(char *dst, const char *src, size_t dstSize)
	{ return StringCat_cc(dst, src, dstSize); }
#endif

/* int StringCompare(a, b) -- a < b --> negative; a == b --> 0; a > b --> positive.
*/
int StringCompare_pp(ConstStr255Param a, ConstStr255Param b);
int StringCompare_pc(ConstStr255Param a, const char *b);
int StringCompare_cp(const char *a, ConstStr255Param b);
#define StringCompare_cc(a, b)  strcmp(a, b)

#ifdef __cplusplus
inline int StringCompare(ConstStr255Param a, ConstStr255Param b)
	{ return StringCompare_pp(a, b); }

inline int StringCompare(ConstStr255Param a, const char *b)
	{ return StringCompare_pc(a, b); }

inline int StringCompare(const char *a, ConstStr255Param b)
	{ return StringCompare_cp(a, b); }
#endif

/* bool StringEqual(a, b) -- true iff string contents are exactly equal.
*/
bool StringEqual_pp(ConstStr255Param a, ConstStr255Param b);
bool StringEqual_pc(ConstStr255Param a, const char *b);
bool StringEqual_cp(const char *a, ConstStr255Param b);
bool StringEqual_cc(const char *a, const char *b);

#ifdef __cplusplus
inline bool StringEqual(ConstStr255Param a, ConstStr255Param b)
	{ return StringEqual_pp(a, b); }

inline bool StringEqual(ConstStr255Param a, const char *b)
	{ return StringEqual_pc(a, b); }

inline bool StringEqual(const char *a, ConstStr255Param b)
	{ return StringEqual_cp(a, b); }

inline bool StringEqual(const char *a, const char *b)
	{ return StringEqual_cc(a, b); }
#endif

/* psprintf -- printf to a Pascal-style string.
*/
int psprintf(StringPtr buf, const char *fmt, ...);
int pvsprintf(StringPtr buf, const char *fmt, va_list arg);

#endif